<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Apilogin extends MX_Controller {

    /**
     * Index Page for this controller and slimmed down API-login page to the GUI.
     * 
     * Note: This doesn't show the login form. It accepts POST requests of UN/PW/URL
     * if the API is enabled. Otherwise it logs errors and redirects to /login.
     */

    // XSS cleaner. 
    //
    // Please note: The regexp are taken from basetypes.schema for the corresponding inputs and conform 
    // with what CODB would accept in the fields 'username' and 'password'.
    public function xssafeLogin($data, $encoding='UTF-8', $type='') {
        if ($type == 'username_field') {
            if (!preg_match('/^[A-Za-z0-9\._-]+$/', $data)) {
                $data = '';
                return $data;
            }
            else {
                return $data;
            }
        }
        elseif ($type == 'password_field') {
            if (!preg_match('/^[^\001-\037\042\177]{6,32}$/', $data)) {
                $data = '';
                return $data;
            }
            else {
                return $data;
            }
        }
        else {
            return htmlspecialchars($data,ENT_QUOTES | ENT_HTML401,$encoding);
        }
    }

    public function index() {
        // We load the BlueOnyx helper library first of all, as we heavily depend on it:
        $this->load->helper('blueonyx');
        init_libraries();

        // Profiling and Benchmarking:
        bx_profiler();

        $CI =& get_instance();

        // Find out if CCEd is running. If it is not, we display an error message and quit:
        if (!$CI->serverScriptHelper) {
            $CI->cceClient = new CceClient();
        }

        // Set up CSRF:
        $csrf = array(
                'name' => $this->security->get_csrf_token_name(),
                'hash' => $this->security->get_csrf_hash()
        );

        // locale and charset setup:
        $ini_langs = initialize_languages(TRUE);
        $locale = $ini_langs['locale'];
        $charset = $ini_langs['charset'];

        // Set cookie for locale if we do NOT have one yet. This HAS to be done here, as the entire
        // i18n she-bang heavily depends on it. 
        $cookie = array('name' => 'locale', 'path' => '/', 'value' => $locale, 'expire' => '31536000');
        $this->input->set_cookie($cookie);

        // Get the IP address of the user accessing the GUI:
        $userip = $this->input->ip_address();

        // Call me a dirty little bastard. But CCEd gets stuck sometimes.
        // Which throws a wrench into the entire GUI. So this is how we
        // handle it: This script tries to establish a connection to CCEd.
        // It has a timeout of five seconds. After which it reports 'TIMEOUT':
        $cce_check = shell_exec('/usr/sausalito/bin/check_cce.pl');
        // If we do have a 'TIMEOUT' or no output at all, then we get down and dirty:
        if (($cce_check == "TIMEOUT") || ($cce_check == "")) {
            // We run the unstuck script vi SUDO. This is the ONLY command that user
            // 'admserv' has sudo capabilities for and it kills off stray CCEd processes,
            // pperld and cced.init. It then does a fast cced.init rehash to get us back up:
            $cce_unstuck = shell_exec('/usr/bin/sudo /usr/sausalito/bin/cced_unstuck.sh');
        }

        // If we get here, CCEd should be running. Either again, or because it was fine.
        if(!$CI->cceClient->connect()) {
            if($locale == "") {
                $this->load->library('System');
                $system = new System();
                $locale = $system->getConfig("defaultLocale");
            }
            $i18n = new I18n("palette", $locale);
        }

        if (!isset($i18n)) {
            $i18n = new I18n("base-alpine", $locale);
        }

        // Get Form data:
        $form_data = $CI->input->post(NULL, TRUE);

        // If we have form data, we sanitize it:
        $attributes = array();
        $ignore_attributes = array();
        if ($CI->input->post(NULL, TRUE)) {
            foreach ($form_data as $key => $value) {
                // Sanitize data received via form fields:
                $form_data[$key] = Apilogin::xssafeLogin($value, 'UTF-8', $key);
            }
            $required_keys = array('username_field', 'password_field', 'secureConnect', 'redirect_target');
            $attributes = GetFormAttributes($i18n, $form_data, $required_keys, $ignore_attributes, $i18n);
            $form_data = $attributes;
        }
        else {
            // Not a POST request. Redirect to /login instead:
            error_log("FAILED API GET ACCESS: IP address: " . $userip);
            delete_cookie("sessionId");
            header('Location: /login');
            exit;
        }

        // Get 'System' object
        $system = $CI->cceClient->getObject('System');
        $API = $CI->cceClient->get($system['OID'], 'API');

        // Is API access allowed?
        $api_access = '0';
        if (isset($API['enabled'])) {
            if ($API['enabled'] == '1') {
                $api_access = '1';
            }
        }
        if ($api_access == '0') {
            // Not a POST request. Redirect to /login instead:
            error_log("ACCESS AGAINST DISABLED API FROM: IP address: " . $userip);
            delete_cookie("sessionId");
            $form_data = array();
            header('Location: /login');
            exit;
        }

        if ((!isset($form_data['username_field'])) || (!isset($form_data['password_field']))) {
            // No login data posted. Redirect to /login instead:
            error_log("FAILED API POST ACCESS: " . json_encode($form_data) . " - IP address: " . $userip);
            delete_cookie("sessionId");
            header('Location: /login');
            exit;
        }

        //
        //-- Form data has been sanitized and validated. Now we check if it matches:
        //

        // Get session ID
        $sessionId = $CI->cceClient->auth($form_data['username_field'], $form_data['password_field']);

        $CI->cceClient->authkey($form_data['username_field'], $sessionId);

        // Get 'System' object
        $system = $CI->cceClient->getObject('System');

        // auth failed?
        if ($sessionId == "") {
            // Login failed. Redirect to /login instead:
            error_log("FAILED API LOGIN: username: " . $form_data['username_field'] . " - IP address: " . $userip);
            delete_cookie("sessionId");
            header('Location: /login');
            exit;
        }
        else {

            // Now if this user is known (and at this point he is), then we check his CODB object to 
            // learn which locale (and charset) he's usually using and use that one instead. After all,
            // it might be different from what his browser tells us:
            $user = $CI->cceClient->getObject("User", array("name" => $form_data['username_field']));

            // Now set the locale based on the users localePreference - if specified and known:
            if ($user['localePreference']) {
                $locale = $user['localePreference'];
            }

            // Cleanup Cache:
            $cap_file_name = "/usr/sausalito/capcache/" . $form_data['username_field'] . "_cap";
            $capabilityGroups_file_name = "/usr/sausalito/capcache/" . $form_data['username_field'] . "_capabilityGroups";
            if (is_file($cap_file_name)) {
              system("rm -f $cap_file_name");
            }
            if (is_file($capabilityGroups_file_name)) {
              system("rm -f $capabilityGroups_file_name");
            }

            // Securely handle 'tzoff' for CSRF timezone offsets. See /usr/sausalito/ui/chorizo/ci/system/core/Security.php for more info.
            $tzoff_file_name = "/usr/sausalito/capcache/" . $form_data['username_field'] . '_' . $sessionId . "_tzoff";
            if (isset($_COOKIE['tzoff'])) {
                if (write_file($tzoff_file_name, $_COOKIE['tzoff'])) {
                  delete_cookie("tzoff");
                }
            }

            // Send cookies that expire at end of the browser session. 
            $this->load->library('encrypt');
            $encrypted_string = $this->encrypt->encode($form_data['password_field']);
            setcookie("XSSkey", $encrypted_string, "0", "/");
            setcookie("loginName", $form_data['username_field'], time()+60*60*24*365, "/");
            setcookie("sessionId", $sessionId, "0", "/");
            setcookie("userip", $userip, "0", "/");
            $cookie = array('name' => 'locale', 'path' => '/', 'value' => $locale, 'expire' => '31536000');
            $this->input->set_cookie($cookie);

            // Manually set CSRF cookie:
            setcookie($CI->BX_SESSION['csrf_cookie_name'], $csrf['hash'], "0", "/");

            //
            //-- Start: Chorizo's Style handling:
            //
  
            // Read the Chorizo's Style from User's CODB object:
            if (isset($user['ChorizoStyle'])) {
                $usersChorizoStyleObject = json_decode(urldecode($user['ChorizoStyle']));
                // Turn Style Object into an Array:
                $usersChorizoStyle = (array) $usersChorizoStyleObject;
            }
            else {
                $usersChorizoStyle = array(
                                          'theme_switcher_php-style' => 'theme_blue.css',
                                          'layout_switcher_php-style' => 'layout_fixed.css',
                                          'nav_switcher_php-style' => 'switcher.css',
                                          'skin_switcher_php-style' => 'skin_light.css',
                                          'bg_switcher_php-style' => 'switcher.css'
                                          );
            }
            // If the user uses a mobile device, we override the fixed layout and switch
            // to the fluid one for a better user-experience:
            if ($CI->agent->is_mobile()) {
                $usersChorizoStyle['layout_switcher_php-style'] = 'layout_fluid.css';
            }
            // Push out cookies for the Users known Style:
            foreach ($usersChorizoStyle as $key => $value) {
                  $theme_cookie = array('name' => $key, 'path' => '/', 'value' => $value, 'expire' => '31536000');
                  $this->input->set_cookie($theme_cookie);
            }

            //
            //-- Redirect to where we're supposed to go:
            //

            if (isset($form_data['redirect_target'])) {
                header('Location: ' . $form_data['redirect_target']);
                exit;
            }
            else {
                // Redirect to GUI:
                header('Location: /gui');
                exit;
            }
        }
    }
}

/*
Copyright (c) 2008-2021 Michael Stauber, SOLARSPEED.NET
Copyright (c) 2008-2021 Team BlueOnyx, BLUEONYX.IT
All Rights Reserved.

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

3. Neither the name of the copyright holder nor the names of its 
   contributors may be used to endorse or promote products derived 
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

You acknowledge that this software is not designed or intended for 
use in the design, construction, operation or maintenance of any 
nuclear facility.

*/
?>